# /u/beebe/src/X79/fold.awk, Fri Jan 11 12:58:41 1991
# Edit by Nelson H. F. Beebe <beebe@000022373100>
# ========================================================================
# Fold a file with long lines, splitting them at white space; runs of
# white space are reduced to single blanks.  Continued lines are marked
# with the command-line SUFFIX string.  If SUFFIX contains backslashes,
# they must be quadrupled to survive.
#
# The default output line width can be changed by setting LINEWIDTH=nn
# on the command line.
#
# The default column widths can be changed by setting COLUMNWIDTH="n1 n2
# ... nk" on the command line; the last width specified applies to all
# remaining fields.  Recommended column widths are n*8-1, since tabs align
# the columns.
#
# Short fields are normally padded with blanks; specify FILLER="..."
# to change that.
#
# The prefix of the continued line can be similarly altered by
# PREFIX="...".
#
# Usage:
#	awk -f fold.awk
#
# [11-Jan-1991]
# ========================================================================
BEGIN {
    if (LINEWIDTH == 0)
	LINEWIDTH = 72;
    LINEWIDTH -= length(SUFFIX);
    if (PREFIX == "")
	PREFIX = "\t\t";
    if (FILLER == "")
	FILLER = "                                                          ";
    if (COLUMNWIDTH != "")
	nw = split(COLUMNWIDTH,width);
    else
	nw = 0;
    for (k = nw + 1; k < 100; ++k)
	width[k] = width[nw];
#    for (k = 1; k < 10; ++k)
#	print "width[" k "] = " width[k];
}

{
    len = 0;
    wc = 0;
    line = "";
    for (k = 1; k <= NF; ++k)
    {
	word = $k;
	if (length(word) < width[k])
	    word = word substr(FILLER,1,width[k] - length(word));
	if ((truelen(line) + length(word)) > LINEWIDTH)
	{
	    out(line,1);
	    line = PREFIX;
	    len = truelen(line);
	    wc = 0;
	}
	line = line ((wc > 0) ? "\t" : "") word;
	wc++;
    }
    out(line,0);
}

function out(s,add_suffix, k)
{
    for (k = length(s); (k > 1) && (substr(line,k,1) == " "); --k)
	;
    printf("%s%s\n", substr(line,1,k), add_suffix ? SUFFIX : "");
}


function truelen(s, k,n)
{
    n = 0;
    for (k = 1; k <= length(s); ++k)
    {
	if (substr(s,k,1) == "\t")
	    n = 8*(int((n + 8)/8));
	else
	    n++;
    }
#   printf("%%%s = TRUELEN[%s]\n",n,s);
    return n;
}
