#include "fpp.h"

#define MATCH(s,n) ((RLENGTH == (n)) && (strnicmp(s,TOKEN,RLENGTH) == 0))

void
pp_line(VOID_ARG)
{
    const char *p;
    const char *thename;
    char  *thevalue;			/* NOT const, because free()'ed */

    if (fpp_debug)
    {
	FPRINTF(stddbg,"DEBUG: Preprocessor line = [%s]\n",fpp_line);
	FFLUSH(stddbg);
    }

    /* isolate the preprocessor command in
       fpp_line[RSTART .. RSTART + RLENGTH - 1] */
    for (p = TOKEN; is_token_letter(*p); ++p) /* TOKEN set in is_pp_line() */
	NOOP;
    RSTART = TOKEN - fpp_line;
    RLENGTH = p - (const char*)TOKEN;

    if (MATCH("if",2))
    {
	level++;
	if (level >= MAXLEVEL)
	    quit("Conditional nesting too deep",1);
	lineno[level] = FNR;		/* for error messages */
	selected[level] = selected[level-1] &&
	    atoi(eval(&fpp_line[RSTART+RLENGTH]));
	branch_done[level] = selected[level];
    }
    else if (MATCH("ifdef",5))
    {
	level++;
	if (level >= MAXLEVEL)
	    quit("Conditional nesting too deep",1);
	lineno[level] = FNR;		/* for error messages */
	selected[level] = selected[level-1] &&
	    (getdef(getname(fpp_line,RSTART+RLENGTH)) == (const char*)NULL)
	    ? 0 : 1;
	branch_done[level] = selected[level];
    }
    else if (MATCH("ifndef",6))
    {
	level++;
	if (level >= MAXLEVEL)
	    quit("Conditional nesting too deep",1);
	lineno[level] = FNR;		/* for error messages */
	selected[level] = selected[level-1] &&
	    (getdef(getname(fpp_line,RSTART+RLENGTH)) == (const char*)NULL)
	    ? 1 : 0;
	branch_done[level] = selected[level];
    }
    else if (MATCH("elseif",6) || MATCH("elif",4))
    {
	if (level <= 0)
	    quit("No preceding #if",1);
	lineno[level] = FNR;		/* for error messages */
	selected[level] = selected[level-1] && !branch_done[level] &&
	    atoi(eval(&fpp_line[RSTART+RLENGTH]));
	if (!branch_done[level])
	    branch_done[level] = selected[level];
    }
    else if (MATCH("else",4))
    {
	if (level <= 0)
	    quit("No preceding #if",1);
	lineno[level] = FNR;		/* for error messages */
	selected[level] = selected[level-1] && !branch_done[level];
    }
    else if (MATCH("endif",5))
    {
	if (level <= 0)
	    quit("No preceding #if",1);
	lineno[level] = FNR;		/* for error messages */
	level--;
    }
    else if (selected[level])
    {
	if (MATCH("define",6))
	{
	    thename = getname(fpp_line,RSTART+RLENGTH);
	    thevalue = trim(&fpp_line[RSTART+RLENGTH]);
	    dodef2(thename,thevalue,0);
	    free(thevalue);
	}
	else if (MATCH("undefine",8) || MATCH("undef",5))
	    doundef(getname(fpp_line,RSTART+RLENGTH));
	else if (MATCH("message",7))
	{
	    thevalue = trim(&fpp_line[RSTART+RLENGTH]);
	    FPRINTF(stderr,"%s%s%s\n",MSG_PREFIX,identification(),thevalue);
	    FFLUSH(stderr);
	    free(thevalue);
	}
	else if (MATCH("error",5))
	{
	    thevalue = trim(&fpp_line[RSTART+RLENGTH]);
	    error(thevalue,0);
	    PRINTF("%s%s%s\n",ERR_PREFIX,identification(),thevalue);
	    free(thevalue);
	}
	else
	{
	    error("Unrecognized preprocessor line",0);
	    PRINTF("%s%sUnrecognized preprocessor line: [%s]\n",
		ERR_PREFIX,identification(),fpp_line);
	}
    }
    print_line(fpp_line);
}
